@@ -65,6 +65,7 @@ enum {
 	PROP_HW_ADDRESS,
 	PROP_COMPANION,
 	PROP_ACTIVE_CHANNEL,
+	PROP_IFINDEX,
 
 	LAST_PROP
 };
@@ -93,6 +94,7 @@ struct _NMDeviceOlpcMeshPrivate
 	gboolean          dispose_has_run;
 
 	struct ether_addr hw_addr;
+	guint32           ifindex;
 
 	GByteArray *      ssid;
 
@@ -192,8 +194,6 @@ nm_device_olpc_mesh_init (NMDeviceOlpcMesh * self)
 	priv->stage1_waiting = FALSE;
 
 	memset (&(priv->hw_addr), 0, sizeof (struct ether_addr));
-
-	nm_device_set_device_type (NM_DEVICE (self), NM_DEVICE_TYPE_OLPC_MESH);
 }
 
 static guint32 iw_freq_to_uint32 (struct iw_freq *freq)
@@ -533,6 +533,14 @@ nm_device_olpc_mesh_set_ssid (NMDeviceOlpcMesh *self, const GByteArray * ssid)
 }
 
 
+guint32
+nm_device_olpc_mesh_get_ifindex (NMDeviceOlpcMesh *self)
+{
+	g_return_val_if_fail (self != NULL, FALSE);
+
+	return NM_DEVICE_OLPC_MESH_GET_PRIVATE (self)->ifindex;
+}
+
 /****************************************************************************/
 
 static void
@@ -675,13 +683,33 @@ get_property (GObject *object, guint prop_id,
 		break;
 	case PROP_COMPANION:
 		if (priv->companion)
-			g_value_set_boxed (value, nm_device_get_udi (priv->companion));
+			g_value_set_boxed (value, nm_device_get_path (priv->companion));
 		else
 			g_value_set_boxed (value, "/");
 		break;
 	case PROP_ACTIVE_CHANNEL:
 		g_value_set_uint (value, nm_device_olpc_mesh_get_channel (device));
 		break;
+	case PROP_IFINDEX:
+		g_value_set_uint (value, nm_device_olpc_mesh_get_ifindex (device));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+set_property (GObject *object, guint prop_id,
+              const GValue *value, GParamSpec *pspec)
+{
+	NMDeviceOlpcMeshPrivate *priv = NM_DEVICE_OLPC_MESH_GET_PRIVATE (object);
+
+	switch (prop_id) {
+	case PROP_IFINDEX:
+		/* construct-only */
+		priv->ifindex = g_value_get_uint (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -698,6 +726,7 @@ nm_device_olpc_mesh_class_init (NMDeviceOlpcMeshClass *klass)
 
 	object_class->constructor = constructor;
 	object_class->get_property = get_property;
+	object_class->set_property = set_property;
 	object_class->dispose = nm_device_olpc_mesh_dispose;
 
 	parent_class->get_type_capabilities = NULL;
@@ -713,7 +742,7 @@ nm_device_olpc_mesh_class_init (NMDeviceOlpcMeshClass *klass)
 
 	parent_class->act_stage1_prepare = real_act_stage1_prepare;
 	parent_class->act_stage2_config = real_act_stage2_config;
-	parent_class->act_stage4_ip_config_timeout = real_act_stage4_ip4_config_timeout;
+	parent_class->act_stage4_ip4_config_timeout = real_act_stage4_ip4_config_timeout;
 
 	/* Properties */
 	g_object_class_install_property
@@ -740,6 +769,13 @@ nm_device_olpc_mesh_class_init (NMDeviceOlpcMeshClass *klass)
 		                   0, G_MAXUINT32, 0,
 		                   G_PARAM_READABLE));
 
+	g_object_class_install_property (object_class, PROP_IFINDEX,
+		g_param_spec_uint (NM_DEVICE_OLPC_MESH_IFINDEX,
+		                   "Ifindex",
+		                   "Interface index",
+		                   0, G_MAXUINT32, 0,
+		                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 	signals[PROPERTIES_CHANGED] =
 		nm_properties_changed_signal_new (object_class,
 		                                  G_STRUCT_OFFSET (NMDeviceOlpcMeshClass, properties_changed));
@@ -884,7 +920,7 @@ check_companion_cb (gpointer user_data)
 	if (priv->device_added_cb != 0)
 		return FALSE;
 
-	manager = nm_manager_get (NULL, FALSE, FALSE);
+	manager = nm_manager_get (NULL, NULL, NULL, FALSE, FALSE, FALSE, NULL);
 
 	priv->device_added_cb = g_signal_connect (manager, "device-added",
 	                                          G_CALLBACK (device_added_cb), self);
@@ -925,11 +961,11 @@ state_changed_cb (NMDevice *device, NMDeviceState state, gpointer user_data)
 }
 
 
-NMDeviceOlpcMesh *
+NMDevice *
 nm_device_olpc_mesh_new (const char *udi,
                          const char *iface,
                          const char *driver,
-                         gboolean managed)
+                         guint32 ifindex)
 {
 	GObject *obj;
 
@@ -941,12 +977,14 @@ nm_device_olpc_mesh_new (const char *udi,
 	                    NM_DEVICE_INTERFACE_UDI, udi,
 	                    NM_DEVICE_INTERFACE_IFACE, iface,
 	                    NM_DEVICE_INTERFACE_DRIVER, driver,
-	                    NM_DEVICE_INTERFACE_MANAGED, managed,
+	                    NM_DEVICE_OLPC_MESH_IFINDEX, ifindex,
+	                    NM_DEVICE_INTERFACE_TYPE_DESC, "802.11 OLPC Mesh",
+	                    NM_DEVICE_INTERFACE_DEVICE_TYPE, NM_DEVICE_TYPE_OLPC_MESH,
 	                    NULL);
 	if (obj == NULL)
 		return NULL;
 
 	g_signal_connect (obj, "state-changed", G_CALLBACK (state_changed_cb), NULL);
 
-	return NM_DEVICE_OLPC_MESH (obj);
+	return NM_DEVICE (obj);
 }
