@@ -34,6 +34,7 @@
 #include <nm-setting-wired.h>
 #include <nm-setting-wireless.h>
 #include <nm-setting-ip4-config.h>
+#include <nm-setting-ip6-config.h>
 
 #include "nm-test-helpers.h"
 
@@ -50,6 +51,7 @@ test_read_valid_wired_connection (void)
 	NMSettingConnection *s_con;
 	NMSettingWired *s_wired;
 	NMSettingIP4Config *s_ip4;
+	NMSettingIP6Config *s_ip6;
 	GError *error = NULL;
 	const GByteArray *array;
 	char expected_mac_address[ETH_ALEN] = { 0x00, 0x11, 0x22, 0x33, 0x44, 0x55 };
@@ -66,6 +68,18 @@ test_read_valid_wired_connection (void)
 	const char *expected_address1_gw = "192.168.0.1";
 	const char *expected_address2_gw = "1.2.1.1";
 	NMIP4Address *ip4_addr;
+	const char *expected6_dns1 = "1111:dddd::aaaa";
+	const char *expected6_dns2 = "1::cafe";
+	const char *expected6_dnssearch1 = "super-domain.com";
+	const char *expected6_dnssearch2 = "redhat.com";
+	const char *expected6_dnssearch3 = "gnu.org";
+	struct in6_addr addr6;
+	const char *expected6_address1 = "abcd:1234:ffff::cdde";
+	const char *expected6_address2 = "1:2:3:4:5:6:7:8";
+	const char *expected6_route_dest = "a:b:c:d::";
+	const char *expected6_route_nh = "f:e:d:c:1:2:3:4";
+	NMIP6Address *ip6_addr;
+	NMIP6Route *ip6_route;
 
 	connection = connection_from_file (TEST_WIRED_FILE);
 	ASSERT (connection != NULL,
@@ -215,7 +229,7 @@ test_read_valid_wired_connection (void)
 	        NM_SETTING_IP4_CONFIG_ADDRESSES);
 
 	ASSERT (nm_ip4_address_get_prefix (ip4_addr) == 24,
-	        "connection-verify-wired", "failed to verify %s: unexpected IP4 address #1 gateway",
+	        "connection-verify-wired", "failed to verify %s: unexpected IP4 address #1 prefix",
 	        TEST_WIRED_FILE,
 	        NM_SETTING_IP4_CONFIG_SETTING_NAME,
 	        NM_SETTING_IP4_CONFIG_ADDRESSES);
@@ -251,7 +265,7 @@ test_read_valid_wired_connection (void)
 	        NM_SETTING_IP4_CONFIG_ADDRESSES);
 
 	ASSERT (nm_ip4_address_get_prefix (ip4_addr) == 16,
-	        "connection-verify-wired", "failed to verify %s: unexpected IP4 address #2 gateway",
+	        "connection-verify-wired", "failed to verify %s: unexpected IP4 address #2 prefix",
 	        TEST_WIRED_FILE,
 	        NM_SETTING_IP4_CONFIG_SETTING_NAME,
 	        NM_SETTING_IP4_CONFIG_ADDRESSES);
@@ -278,6 +292,172 @@ test_read_valid_wired_connection (void)
 	        NM_SETTING_IP4_CONFIG_SETTING_NAME,
 	        NM_SETTING_IP4_CONFIG_ADDRESSES);
 
+	/* ===== IPv6 SETTING ===== */
+
+	s_ip6 = NM_SETTING_IP6_CONFIG (nm_connection_get_setting (connection, NM_TYPE_SETTING_IP6_CONFIG));
+	ASSERT (s_ip6 != NULL,
+	        "connection-verify-ip6", "failed to verify %s: missing %s setting",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME);
+
+	/* Method */
+	tmp = nm_setting_ip6_config_get_method (s_ip6);
+	ASSERT (strcmp (tmp, NM_SETTING_IP6_CONFIG_METHOD_MANUAL) == 0,
+	        "connection-verify-wired", "failed to verify %s: unexpected %s / %s key value",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_METHOD);
+
+	/* DNS Addresses */
+	ASSERT (nm_setting_ip6_config_get_num_dns (s_ip6) == 2,
+	        "connection-verify-wired", "failed to verify %s: unexpected %s / %s key value",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS);
+
+	ASSERT (inet_pton (AF_INET6, expected6_dns1, &addr6) > 0,
+	        "connection-verify-wired", "failed to verify %s: couldn't convert DNS IP6 address #1",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS);
+	ASSERT (IN6_ARE_ADDR_EQUAL (nm_setting_ip6_config_get_dns (s_ip6, 0), &addr6),
+	        "connection-verify-wired", "failed to verify %s: unexpected %s / %s key value #1",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS);
+
+	ASSERT (inet_pton (AF_INET6, expected6_dns2, &addr6) > 0,
+	        "connection-verify-wired", "failed to verify %s: couldn't convert DNS IP address #2",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS);
+	ASSERT (IN6_ARE_ADDR_EQUAL (nm_setting_ip6_config_get_dns (s_ip6, 1), &addr6),
+	        "connection-verify-wired", "failed to verify %s: unexpected %s / %s key value #2",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS);
+
+	ASSERT (nm_setting_ip6_config_get_num_addresses (s_ip6) == 2,
+	        "connection-verify-wired", "failed to verify %s: unexpected %s / %s key value",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS);
+
+	/* DNS Searches */
+	ASSERT (nm_setting_ip6_config_get_num_dns_searches (s_ip6) == 3,
+	        "connection-verify-wired", "failed to verify %s: unexpected %s / %s key value",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS_SEARCH);
+
+	ASSERT (!strcmp (nm_setting_ip6_config_get_dns_search (s_ip6, 0), expected6_dnssearch1),
+	        "connection-verify-wired", "failed to verify %s: unexpected %s / %s key value #1",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS_SEARCH);
+	ASSERT (!strcmp (nm_setting_ip6_config_get_dns_search (s_ip6, 1), expected6_dnssearch2),
+	        "connection-verify-wired", "failed to verify %s: unexpected %s / %s key value #2",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS_SEARCH);
+	ASSERT (!strcmp (nm_setting_ip6_config_get_dns_search (s_ip6, 2), expected6_dnssearch3),
+	        "connection-verify-wired", "failed to verify %s: unexpected %s / %s key value #3",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS_SEARCH);
+
+	/* Address #1 */
+	ip6_addr = nm_setting_ip6_config_get_address (s_ip6, 0);
+	ASSERT (ip6_addr,
+	        "connection-verify-wired", "failed to verify %s: missing IP6 address #1",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_ADDRESSES);
+
+	ASSERT (nm_ip6_address_get_prefix (ip6_addr) == 64,
+	        "connection-verify-wired", "failed to verify %s: unexpected IP6 address #1 prefix",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_ADDRESSES);
+
+	ASSERT (inet_pton (AF_INET6, expected6_address1, &addr6) > 0,
+	        "connection-verify-wired", "failed to verify %s: couldn't convert IP address #1",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS);
+	ASSERT (IN6_ARE_ADDR_EQUAL (nm_ip6_address_get_address (ip6_addr), &addr6),
+	        "connection-verify-wired", "failed to verify %s: unexpected IP4 address #1",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_ADDRESSES);
+
+	/* Address #2 */
+	ip6_addr = nm_setting_ip6_config_get_address (s_ip6, 1);
+	ASSERT (ip6_addr,
+	        "connection-verify-wired", "failed to verify %s: missing IP6 address #2",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_ADDRESSES);
+
+	ASSERT (nm_ip6_address_get_prefix (ip6_addr) == 96,
+	        "connection-verify-wired", "failed to verify %s: unexpected IP6 address #2 prefix",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_ADDRESSES);
+
+	ASSERT (inet_pton (AF_INET6, expected6_address2, &addr6) > 0,
+	        "connection-verify-wired", "failed to verify %s: couldn't convert IP address #2",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS);
+	ASSERT (IN6_ARE_ADDR_EQUAL (nm_ip6_address_get_address (ip6_addr), &addr6),
+	        "connection-verify-wired", "failed to verify %s: unexpected IP6 address #2",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_ADDRESSES);
+
+	/* Route #1 */
+	ip6_route = nm_setting_ip6_config_get_route (s_ip6, 0);
+	ASSERT (ip6_route,
+	        "connection-verify-wired", "failed to verify %s: missing IP6 route #1",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_ROUTES);
+
+	ASSERT (inet_pton (AF_INET6, expected6_route_dest, &addr6) > 0,
+	        "connection-verify-wired", "failed to verify %s: couldn't convert IP route dest #1",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS);
+	ASSERT (IN6_ARE_ADDR_EQUAL (nm_ip6_route_get_dest (ip6_route), &addr6),
+	        "connection-verify-wired", "failed to verify %s: unexpected IP4 route dest #1",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_ROUTES);
+
+	ASSERT (nm_ip6_route_get_prefix (ip6_route) == 64,
+	        "connection-verify-wired", "failed to verify %s: unexpected IP6 route #1 prefix",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_ROUTES);
+
+	ASSERT (inet_pton (AF_INET6, expected6_route_nh, &addr6) > 0,
+	        "connection-verify-wired", "failed to verify %s: couldn't convert IP route next hop #1",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_DNS);
+	ASSERT (IN6_ARE_ADDR_EQUAL (nm_ip6_route_get_next_hop (ip6_route), &addr6),
+	        "connection-verify-wired", "failed to verify %s: unexpected IP4 route dest #1",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_ROUTES);
+
+	ASSERT (nm_ip6_route_get_metric (ip6_route) == 99,
+	        "connection-verify-wired", "failed to verify %s: unexpected IP6 route #1 metric",
+	        TEST_WIRED_FILE,
+	        NM_SETTING_IP6_CONFIG_SETTING_NAME,
+	        NM_SETTING_IP6_CONFIG_ROUTES);
+
 	g_object_unref (connection);
 }
 
@@ -327,6 +507,49 @@ add_one_ip4_route (NMSettingIP4Config *s_ip4,
 	nm_ip4_route_unref (route);
 }
 
+static void
+add_one_ip6_address (NMSettingIP6Config *s_ip6,
+                     const char *addr,
+                     guint32 prefix)
+{
+	struct in6_addr tmp;
+	NMIP6Address *ip6_addr;
+
+	ip6_addr = nm_ip6_address_new ();
+	nm_ip6_address_set_prefix (ip6_addr, prefix);
+
+	inet_pton (AF_INET6, addr, &tmp);
+	nm_ip6_address_set_address (ip6_addr, &tmp);
+
+	nm_setting_ip6_config_add_address (s_ip6, ip6_addr);
+	nm_ip6_address_unref (ip6_addr);
+}
+
+static void
+add_one_ip6_route (NMSettingIP6Config *s_ip6,
+                   const char *dest,
+                   const char *nh,
+                   guint32 prefix,
+                   guint32 metric)
+{
+	struct in6_addr addr;
+	NMIP6Route *route;
+
+	route = nm_ip6_route_new ();
+	nm_ip6_route_set_prefix (route, prefix);
+	nm_ip6_route_set_metric (route, metric);
+
+	inet_pton (AF_INET6, dest, &addr);
+	nm_ip6_route_set_dest (route, &addr);
+
+	inet_pton (AF_INET6, nh, &addr);
+	nm_ip6_route_set_next_hop (route, &addr);
+
+	nm_setting_ip6_config_add_route (s_ip6, route);
+	nm_ip6_route_unref (route);
+}
+
+
 static void
 test_write_wired_connection (void)
 {
@@ -334,6 +557,7 @@ test_write_wired_connection (void)
 	NMSettingConnection *s_con;
 	NMSettingWired *s_wired;
 	NMSettingIP4Config *s_ip4;
+	NMSettingIP6Config *s_ip6;
 	char *uuid;
 	GByteArray *mac;
 	unsigned char tmpmac[] = { 0x99, 0x88, 0x77, 0x66, 0x55, 0x44 };
@@ -344,6 +568,7 @@ test_write_wired_connection (void)
 	pid_t owner_grp;
 	uid_t owner_uid;
 	struct in_addr addr;
+	struct in6_addr addr6;
 	const char *dns1 = "4.2.2.1";
 	const char *dns2 = "4.2.2.2";
 	const char *address1 = "192.168.0.5";
@@ -354,6 +579,14 @@ test_write_wired_connection (void)
 	const char *route1_nh = "10.10.10.1";
 	const char *route2 = "0.0.0.0";
 	const char *route2_nh = "1.2.1.1";
+	const char *dns6_1 = "1::cafe";
+	const char *dns6_2 = "2::cafe";
+	const char *address6_1 = "abcd::beef";
+	const char *address6_2 = "dcba::beef";
+	const char *route6_1 = "1:2:3:4:5:6:7:8";
+	const char *route6_1_nh = "8:7:6:5:4:3:2:1";
+	const char *route6_2 = "::";
+	const char *route6_2_nh = "2001::1111";
 	guint64 timestamp = 0x12345678L;
 
 	connection = nm_connection_new ();
@@ -420,6 +653,35 @@ test_write_wired_connection (void)
 	inet_pton (AF_INET, dns2, &addr);
 	nm_setting_ip4_config_add_dns (s_ip4, addr.s_addr);
 
+	/* IP6 setting */
+
+	s_ip6 = NM_SETTING_IP6_CONFIG (nm_setting_ip6_config_new ());
+	ASSERT (s_ip6 != NULL,
+			"connection-write", "failed to allocate new %s setting",
+			NM_SETTING_IP6_CONFIG_SETTING_NAME);
+	nm_connection_add_setting (connection, NM_SETTING (s_ip6));
+
+	g_object_set (s_ip6,
+	              NM_SETTING_IP6_CONFIG_METHOD, NM_SETTING_IP6_CONFIG_METHOD_MANUAL,
+	              NULL);
+
+	/* Addresses */
+	add_one_ip6_address (s_ip6, address6_1, 64);
+	add_one_ip6_address (s_ip6, address6_2, 56);
+
+	/* Routes */
+	add_one_ip6_route (s_ip6, route6_1, route6_1_nh, 64, 3);
+	add_one_ip6_route (s_ip6, route6_2, route6_2_nh, 56, 1);
+
+	/* DNS servers */
+	inet_pton (AF_INET6, dns6_1, &addr6);
+	nm_setting_ip6_config_add_dns (s_ip6, &addr6);
+	inet_pton (AF_INET6, dns6_2, &addr6);
+	nm_setting_ip6_config_add_dns (s_ip6, &addr6);
+
+	/* DNS searches */
+	nm_setting_ip6_config_add_dns_search (s_ip6, "wallaceandgromit.com");
+
 	/* Write out the connection */
 	owner_uid = geteuid ();
 	owner_grp = getegid ();
@@ -570,6 +832,7 @@ test_write_wireless_connection (void)
 	NMSettingConnection *s_con;
 	NMSettingWireless *s_wireless;
 	NMSettingIP4Config *s_ip4;
+	NMSettingIP6Config *s_ip6;
 	char *uuid;
 	GByteArray *bssid;
 	unsigned char tmpbssid[] = { 0xaa, 0xb9, 0xa1, 0x74, 0x55, 0x44 };
@@ -640,6 +903,18 @@ test_write_wireless_connection (void)
 	              NM_SETTING_IP4_CONFIG_METHOD, NM_SETTING_IP4_CONFIG_METHOD_AUTO,
 	              NULL);
 
+	/* IP6 setting */
+
+	s_ip6 = NM_SETTING_IP6_CONFIG (nm_setting_ip6_config_new ());
+	ASSERT (s_ip6 != NULL,
+			"connection-write", "failed to allocate new %s setting",
+			NM_SETTING_IP6_CONFIG_SETTING_NAME);
+	nm_connection_add_setting (connection, NM_SETTING (s_ip6));
+
+	g_object_set (s_ip6,
+	              NM_SETTING_IP6_CONFIG_METHOD, NM_SETTING_IP6_CONFIG_METHOD_AUTO,
+	              NULL);
+
 	/* Write out the connection */
 	owner_uid = geteuid ();
 	owner_grp = getegid ();
@@ -670,7 +945,7 @@ int main (int argc, char **argv)
 {
 	GError *error = NULL;
 	DBusGConnection *bus;
-	char *basename;
+	char *base;
 
 	g_type_init ();
 	bus = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
@@ -685,9 +960,10 @@ int main (int argc, char **argv)
 	test_read_valid_wireless_connection ();
 	test_write_wireless_connection ();
 
-	basename = g_path_get_basename (argv[0]);
-	fprintf (stdout, "%s: SUCCESS\n", basename);
-	g_free (basename);
+	base = g_path_get_basename (argv[0]);
+	fprintf (stdout, "%s: SUCCESS\n", base);
+	g_free (base);
+	dbus_g_connection_unref (bus);
 	return 0;
 }
 
